@@ -2,12 +2,10 @@
 # import sys
 import numpy as np
 import pandas as pd
-import matplotlib.pyplot as plt
 from sklearn.svm import SVC
 from sklearn.metrics import classification_report
 from ..utils.base import save_fig
 from ..global_variable import MODEL_OUTPUT_IMAGE_PATH
-from sklearn import tree
 from sklearn.tree import DecisionTreeClassifier
 from sklearn.ensemble import RandomForestClassifier
 import xgboost
@@ -16,7 +14,8 @@ from typing import Union, Optional, List, Dict, Callable, Tuple, Any, Sequence,
 from multipledispatch import dispatch
 from flaml import AutoML
 from ._base import WorkflowBase
-from .func.algo_classification._common import confusion_matrix_plot, contour_data
+from .func.algo_classification._common import confusion_matrix_plot, contour_data, plot_precision_recall, plot_ROC,\
+    cross_validation
 from .func.algo_classification._svm import plot_2d_decision_boundary
 from .func.algo_classification._xgboost import feature_importance_map, feature_importance_value, \
     feature_weights_histograms
@@ -28,7 +27,7 @@ from .func.algo_classification._rf import feature_importances
 class ClassificationWorkflowBase(WorkflowBase):
     """The base workflow class of classification algorithms."""
 
-    common_function = ["Model Score", "Confusion Matrix"]
+    common_function = ["Model Score", "Confusion Matrix", 'Cross Validation', 'Model Prediction', 'Model Persistence']
 
     def __init__(self) -> None:
         super().__init__()
@@ -83,6 +82,12 @@ class ClassificationWorkflowBase(WorkflowBase):
         print("-----* Model Score *-----")
         print(classification_report(y_true, y_predict))
 
+    @staticmethod
+    def _cross_validation(trained_model: object, X_train: pd.DataFrame, y_train: pd.DataFrame, cv_num: int = 10) -> None:
+        print("-----* Cross Validation *-----")
+        print(f"K-Folds: {cv_num}")
+        cross_validation(trained_model, X_train, y_train, cv_num=cv_num)
+
     @staticmethod
     def _confusion_matrix_plot(y_test: pd.DataFrame, y_test_predict: pd.DataFrame,
                                trained_model: object, algorithm_name: str, store_path: str) -> None:
@@ -99,6 +104,7 @@ class ClassificationWorkflowBase(WorkflowBase):
     def common_components(self) -> None:
         """Invoke all common application functions for classification algorithms by Scikit-learn framework."""
         self._score(ClassificationWorkflowBase.y_test, ClassificationWorkflowBase.y_test_predict)
+        self._cross_validation(self.model, ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train, 10)
         self._confusion_matrix_plot(ClassificationWorkflowBase.y_test, ClassificationWorkflowBase.y_test_predict,
                                     self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
@@ -106,6 +112,7 @@ class ClassificationWorkflowBase(WorkflowBase):
     def common_components(self, is_automl: bool) -> None:
         """Invoke all common application functions for classification algorithms by FLAML framework."""
         self._score(ClassificationWorkflowBase.y_test, ClassificationWorkflowBase.y_test_predict)
+        self._cross_validation(self.auto_model, ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train, 10)
         self._confusion_matrix_plot(ClassificationWorkflowBase.y_test, ClassificationWorkflowBase.y_test_predict,
                                     self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
@@ -114,7 +121,7 @@ class SVMClassification(ClassificationWorkflowBase):
     """The automation workflow of using SVC algorithm to make insightful products."""
 
     name = "Support Vector Machine"
-    special_function = ['Two-dimensional Decision Boundary Diagram']
+    special_function = ['Two-dimensional Decision Boundary Diagram', 'Precision_Recall_Curve', 'ROC Curve']
 
     def __init__(
             self,
@@ -311,32 +318,6 @@ class SVMClassification(ClassificationWorkflowBase):
 
         return MySVMClassification
 
-    # TODO(Sany sanyhew1097618435@163.com): think about What layout of the graph is more user-friendly.
-    """
-    def plot_svc_surface_function(self):
-        # divide the two selected elements and draw an image
-        print("-----* Two-dimensional Decision Surface Boundary Diagram *-----")
-        plt.figure()
-        y = np.array(ClassificationWorkflowBase().y)
-        X = np.array(ClassificationWorkflowBase().X)
-        X = PCA(n_components=2).fit_transform(X)
-        y = np.squeeze(y)
-        clf = self.model.fit(X, y)
-        plt.scatter(X[:, 0], X[:, 1], c=y, s=50, cmap=ListedColormap(['#FF0000', '#0000FF']),alpha=0.6)
-        ax = plt.gca()
-        xlim = ax.get_xlim()
-        ylim = ax.get_ylim()
-        x = np.linspace(xlim[0], xlim[1], 30)
-        y = np.linspace(ylim[0], ylim[1], 30)
-        Y, X = np.meshgrid(y, x)
-        Z = clf.decision_function(np.c_[X.ravel(), Y.ravel()])
-        Z = Z.reshape(X.shape)
-        ax.contourf(X, Y, Z, cmap=plt.cm.RdYlBu, alpha=0.5)
-        ax.set_xlim(xlim)
-        ax.set_ylim(ylim)
-        save_fig('SVC Surface Function Plot', MODEL_OUTPUT_IMAGE_PATH)
-    """
-
     @staticmethod
     def _plot_2d_decision_boundary(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: Any,
                                    image_config: dict, algorithm_name: str, store_path: str,
@@ -345,11 +326,29 @@ class SVMClassification(ClassificationWorkflowBase):
         """Plot the decision boundary of the trained model with the testing data set below."""
         print("-----* Two-dimensional Decision Boundary Diagram *-----")
         plot_2d_decision_boundary(X, X_test, y_test, trained_model, image_config, algorithm_name)
-        save_fig(f'2D Decision Boundary - {algorithm_name}', store_path)
+        save_fig(f'Classification - {algorithm_name} - Decision Boundary', store_path)
+
+    @staticmethod
+    def _plot_precision_recall( X_train: pd.DataFrame, y_train: pd.DataFrame,
+                                trained_model: object, algorithm_name: str, store_path: str) -> None:
+        print("-----* Precision and Recall Versus the Decision Threshold *-----")
+        plot_precision_recall(X_train, y_train, trained_model, algorithm_name)
+        save_fig(f'Precision_Recall_Curve - {algorithm_name}', store_path)
+
+    @staticmethod
+    def _plot_ROC(X_train: pd.DataFrame, y_train: pd.DataFrame,
+                  trained_model: object, algorithm_name: str, store_path: str) -> None:
+        print("-----* ROC Curve *-----")
+        plot_ROC(X_train, y_train, trained_model, algorithm_name)
+        save_fig(f'ROC_Curve - {algorithm_name}', store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
+        self._plot_precision_recall(ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train,
+                                    self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_ROC(ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train,
+                       self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         if SVMClassification.X.shape[1] == 2:
             self._plot_2d_decision_boundary(SVMClassification.X, SVMClassification.X_test, SVMClassification.y_test,
                                             self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
@@ -365,6 +364,10 @@ class SVMClassification(ClassificationWorkflowBase):
     @dispatch(bool)
     def special_components(self, is_automl: bool, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
+        self._plot_precision_recall(ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train,
+                                    self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_ROC(ClassificationWorkflowBase.X_train, ClassificationWorkflowBase.y_train,
+                       self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         if SVMClassification.X.shape[1] == 2:
             self._plot_2d_decision_boundary(SVMClassification.X, SVMClassification.X_test, SVMClassification.y_test,
                                             self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
@@ -550,53 +553,28 @@ class DecisionTreeClassification(ClassificationWorkflowBase):
                                             ccp_alpha=self.ccp_alpha)
         self.naming = DecisionTreeClassification.name
 
-    def plot_tree_function(self, cls: object, trained_model: any, algorithm_name: str, store_path: str) -> None:
-        # Drawing decision tree diagrams
+    def plot_tree_function(self, trained_model: object, image_config: dict, algorithm_name: str, store_path: str) -> None:
+        """Drawing decision tree diagrams."""
         print("-----* Decision Tree Plot *-----")
-        decision_tree_plot(cls, trained_model, algorithm_name)
-        save_fig('Decision Tree Classification Plot', store_path)
-
-    '''
-       def decision_surface_plot(self):
-           #############################################################
-           #Plot the decision surfaces of forests of the data
-           #############################################################
-           print("Decision_Surface_Plot", "Drawing Decision Surface Plot")
-           plt.figure()
-           y = np.array(ClassificationWorkflowBase().y)
-           X = np.array(ClassificationWorkflowBase().X)
-           X = PCA(n_components=2).fit_transform(X)
-           self.model.fit(X,y)
-           x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
-           y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
-           xx, yy = np.meshgrid(np.arange(x_min, x_max, 0.02), np.arange(y_min, y_max, 0.02))
-           Z = self.model.predict(np.c_[xx.ravel(), yy.ravel()])
-           Z = Z.reshape(xx.shape)
-           plt.contour(xx, yy, Z, colors="k", levels=[-1, 0, 1], alpha=0.5, linestyles=["--", "-", "--"], s=2)
-           plt.contourf(xx, yy, Z, cmap=plt.cm.RdYlBu, alpha=0.5)
-           plt.scatter(X[:, 0], X[:, 1],c=y,cmap=ListedColormap(['#FF0000', '#0000FF']),alpha=0.6,s=20)
-           plt.suptitle("Decision Surface Plot ", fontsize=12)
-           plt.axis("tight")
-           plt.tight_layout(h_pad=0.2, w_pad=0.2, pad=2.5)
-           save_fig('Decision Surface Plot', MODEL_OUTPUT_IMAGE_PATH)
-       '''
+        decision_tree_plot(trained_model, image_config)
+        save_fig(f"Classification - {algorithm_name} - Tree Graph", store_path)
 
     @staticmethod
-    def dt_plot_2d_decision_boundary(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: Any,
-                                     algorithm_name: str, store_path: str,
-                                     contour_data: Optional[List[np.ndarray]] = None,
-                                     labels: Optional[np.ndarray] = None) -> None:
+    def _plot_2d_decision_boundary(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: Any,
+                                   image_config: dict, algorithm_name: str, store_path: str,
+                                   contour_data: Optional[List[np.ndarray]] = None,
+                                   labels: Optional[np.ndarray] = None) -> None:
         """Plot the decision boundary of the trained model with the testing data set below."""
         print("-----* Two-dimensional Decision Boundary Diagram *-----")
-        plot_2d_decision_boundary(X, X_test, y_test, trained_model, algorithm_name)
-        save_fig(f'2d Decision Boundary - {algorithm_name}', store_path)
+        plot_2d_decision_boundary(X, X_test, y_test, trained_model, image_config, algorithm_name)
+        save_fig(f'Classification - {algorithm_name} - Decision Boundary', store_path)
 
     def special_components(self, **kwargs) -> None:
-        self.plot_tree_function(ClassificationWorkflowBase, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self.plot_tree_function(self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         if DecisionTreeClassification.X.shape[1] == 2:
-            self.dt_plot_2d_decision_boundary(DecisionTreeClassification.X, DecisionTreeClassification.X_test,
+            self._plot_2d_decision_boundary(DecisionTreeClassification.X, DecisionTreeClassification.X_test,
                                               DecisionTreeClassification.y_test,
-                                              self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+                                              self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
 
 class RandomForestClassification(ClassificationWorkflowBase):
@@ -868,29 +846,28 @@ class RandomForestClassification(ClassificationWorkflowBase):
         save_fig(f"Feature Importance - {algorithm_name}", store_path)
 
     @staticmethod
-    def _tree_plot(trained_model: object, algorithm_name: str, store_path: str) -> None:
+    def _tree_plot(trained_model: object, image_config: dict, algorithm_name: str, store_path: str) -> None:
         """Draw a diagram of the first decision tree of the forest."""
         print("-----* Random Forest's Tree Plot *-----")
-        plt.figure()
-        tree.plot_tree(trained_model.estimators_[0])
-        save_fig(f"First Decision Tree - {algorithm_name}", store_path)
+        decision_tree_plot(trained_model.estimators_[0], image_config)
+        save_fig(f"Classification - {algorithm_name} - Tree Graph", store_path)
 
     @staticmethod
     def _plot_2d_decision_boundary(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: object,
-                                   algorithm_name: str, store_path: str) -> None:
+                                   image_config: dict, algorithm_name: str, store_path: str) -> None:
         """Plot the decision boundary of the trained model with the testing data set below."""
         print("-----* Two-dimensional Decision Boundary Diagram *-----")
-        plot_2d_decision_boundary(X, X_test, y_test, trained_model, algorithm_name)
-        save_fig(f'2D Decision Boundary - {algorithm_name}', store_path)
+        plot_2d_decision_boundary(X, X_test, y_test, trained_model, image_config, algorithm_name)
+        save_fig(f"Classification - {algorithm_name} - Decision Boundary", store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
         self._feature_importances(RandomForestClassification.X_train, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
-        self._tree_plot(self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._tree_plot(self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         if RandomForestClassification.X.shape[1] == 2:
             self._plot_2d_decision_boundary(RandomForestClassification.X, RandomForestClassification.X_test,
-                                            RandomForestClassification.y_test, self.model, self.naming,
+                                            RandomForestClassification.y_test, self.model, self.image_config, self.naming,
                                             MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
@@ -898,10 +875,10 @@ class RandomForestClassification(ClassificationWorkflowBase):
         """Invoke all special application functions for this algorithms by FLAML framework."""
         self._feature_importances(RandomForestClassification.X_train, self.auto_model, self.naming,
                                   MODEL_OUTPUT_IMAGE_PATH)
-        self._tree_plot(self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._tree_plot(self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         if RandomForestClassification.X.shape[1] == 2:
             self._plot_2d_decision_boundary(RandomForestClassification.X, RandomForestClassification.X_test,
-                                            RandomForestClassification.y_test, self.auto_model, self.naming,
+                                            RandomForestClassification.y_test, self.auto_model, self.image_config, self.naming,
                                             MODEL_OUTPUT_IMAGE_PATH)
 
 
@@ -950,7 +927,7 @@ class XgboostClassification(ClassificationWorkflowBase):
             gpu_id: Optional[int] = None,
             validate_parameters: Optional[bool] = None,
             predictor: Optional[str] = None,
-            enable_categorical: bool = False,
+            # enable_categorical: bool = False,
             eval_metric: Optional[Union[str, List[str], Callable]] = None,
             early_stopping_rounds: Optional[int] = None,
             **kwargs: Any
@@ -987,7 +964,7 @@ class XgboostClassification(ClassificationWorkflowBase):
         self.gpu_id = gpu_id
         self.validate_parameters = validate_parameters
         self.predictor = predictor
-        self.enable_categorical = enable_categorical
+        # self.enable_categorical = enable_categorical
         self.eval_metric = eval_metric
         self.early_stopping_rounds = early_stopping_rounds
         if kwargs:
@@ -1024,7 +1001,7 @@ class XgboostClassification(ClassificationWorkflowBase):
             gpu_id=self.gpu_id,
             validate_parameters=self.validate_parameters,
             predictor=self.predictor,
-            enable_categorical=self.enable_categorical,
+            # enable_categorical=self.enable_categorical,
             eval_metric=self.eval_metric,
             early_stopping_rounds=self.early_stopping_rounds,
         )
@@ -1044,24 +1021,24 @@ class XgboostClassification(ClassificationWorkflowBase):
         return configuration
 
     @staticmethod
-    def _feature_importance_series(data: pd.DataFrame, trained_model: any, algorithm_name: str,
+    def _feature_importance_series(data: pd.DataFrame, trained_model: any, algorithm_name: str, image_config: dict,
                                    store_path: str) -> None:
         print("-----* Feature Importance *-----")
         feature_importance_value(data, trained_model)
-        feature_weights_histograms(data, trained_model, algorithm_name)
-        save_fig(f"Feature Weights - Histograms Plot - {algorithm_name}", store_path)
-        feature_importance_map(trained_model, algorithm_name)
-        save_fig(f"Feature Importance Map Plot - {algorithm_name}", store_path)
+        feature_weights_histograms(trained_model, image_config, algorithm_name)
+        save_fig(f"Classification - {algorithm_name} - Feature Weights Histograms Plot", store_path)
+        feature_importance_map(trained_model, image_config,algorithm_name)
+        save_fig(f"Classification - {algorithm_name} - Feature Importance Map Plot", store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
-        self._feature_importance_series(XgboostClassification.X, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance_series(self.X, self.model, self.naming, self.image_config, MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
-        self._feature_importance_series(XgboostClassification.X, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance_series(XgboostClassification.X, self.auto_model, self.naming, self.image_config, MODEL_OUTPUT_IMAGE_PATH)
 
     # def plot(self):
     # TODO(solve the problem of failed to execute WindowsPath('dot'), make sure the Graphviz executables are on your systems' PATH
@@ -1083,7 +1060,7 @@ class LogisticRegressionClassification(ClassificationWorkflowBase):
     """The automation workflow of using Logistic Regression algorithm to make insightful products."""
 
     name = "Logistic Regression"
-    special_function = ['Feature Importance']
+    special_function = ['Feature Importance', 'Precision_Recall_Curve', 'ROC Curve']
 
     def __init__(
             self,
@@ -1304,18 +1281,41 @@ class LogisticRegressionClassification(ClassificationWorkflowBase):
         }
         return configuration
 
-    def _feature_importance(self, data: pd.DataFrame, trained_model: any, algorithm_name: str, store_path: str) -> None:
+    @staticmethod
+    def _feature_importance(data: pd.DataFrame, trained_model: any, algorithm_name: str, store_path: str) -> None:
         """Print the feature coefficient value orderly."""
         print("-----* Feature Importance *-----")
         logistic_importance_plot(data, trained_model, algorithm_name)
         save_fig("LogisticRegression_feature_importance", store_path)
 
+    @staticmethod
+    def _plot_precision_recall(X_train: pd.DataFrame, y_train: pd.DataFrame,
+                               trained_model: object, algorithm_name: str, store_path: str) -> None:
+        print("-----* Precision and Recall Versus the Decision Threshold *-----")
+        plot_precision_recall(X_train, y_train, trained_model, algorithm_name)
+        save_fig(f'Precision_Recall_Curve - {algorithm_name}', store_path)
+
+    @staticmethod
+    def _plot_ROC(X_train: pd.DataFrame, y_train: pd.DataFrame,
+                  trained_model: object, algorithm_name: str, store_path: str) -> None:
+        print("-----* ROC Curve *-----")
+        plot_ROC(X_train, y_train, trained_model, algorithm_name)
+        save_fig(f'ROC_Curve - {algorithm_name}', store_path)
+
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
-        self._feature_importance(ClassificationWorkflowBase.X, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance(LogisticRegressionClassification.X, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_precision_recall(LogisticRegressionClassification.X_train, LogisticRegressionClassification.y_train,
+                                    self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_ROC(LogisticRegressionClassification.X_train, LogisticRegressionClassification.y_train,
+                       self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
-        self._feature_importance(ClassificationWorkflowBase.X, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance(LogisticRegressionClassification.X, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_precision_recall(LogisticRegressionClassification.X_train, LogisticRegressionClassification.y_train,
+                                    self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._plot_ROC(LogisticRegressionClassification.X_train, LogisticRegressionClassification.y_train,
+                       self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
\ No newline at end of file
