@@ -9,7 +9,6 @@ from sklearn.svm import SVR
 from sklearn.neural_network import MLPRegressor
 from typing import Union, Optional, List, Dict, Callable, Tuple, Any
 from typing import Sequence
-import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import xgboost
@@ -18,16 +17,17 @@ from flaml import AutoML
 from ._base import WorkflowBase
 from .func.algo_regression._common import plot_predicted_value_evaluation, plot_true_vs_predicted, score, cross_validation
 from .func.algo_regression._polynomial import show_formula
-from .func.algo_regression._rf import feature_importance__
+from .func.algo_regression._rf import feature_importance__, box_plot
 from .func.algo_regression._xgboost import feature_importance, histograms_feature_weights, permutation_importance_
 from .func.algo_regression._linear import show_formula, plot_2d_graph, plot_3d_graph
 from .func.algo_regression._extra_tree import feature_importances
+from .func.algo_regression._decision_tree import decision_tree_plot
 
 
 class RegressionWorkflowBase(WorkflowBase):
     """The base workflow class of regression algorithms."""
 
-    common_function = ['Model Score', 'Cross Validation']
+    common_function = ['Model Score', 'Cross Validation', 'Model Prediction', 'Model Persistence']
 
     def __init__(self) -> None:
         super().__init__()
@@ -114,6 +114,7 @@ class RegressionWorkflowBase(WorkflowBase):
     @staticmethod
     def _cross_validation(trained_model: object, X_train: pd.DataFrame, y_train: pd.DataFrame, cv_num: int = 10) -> None:
         print("-----* Cross Validation *-----")
+        print(f"K-Folds: {cv_num}")
         cross_validation(trained_model, X_train, y_train, cv_num=cv_num)
 
     # TODO(Sany sanyhew1097618435@163.com): How to prevent overfitting
@@ -243,7 +244,7 @@ class XgboostRegression(RegressionWorkflowBase):
                 gpu_id: Optional[int] = None,
                 validate_parameters: Optional[bool] = None,
                 predictor: Optional[str] = None,
-                enable_categorical: bool = False,
+                #enable_categorical: bool = False,
                 eval_metric: Optional[Union[str, List[str], Callable]] = None,
                 early_stopping_rounds: Optional[int] = None,
                 **kwargs: Any) -> None:
@@ -445,7 +446,7 @@ class XgboostRegression(RegressionWorkflowBase):
         self.gpu_id = gpu_id
         self.validate_parameters = validate_parameters
         self.predictor = predictor
-        self.enable_categorical = enable_categorical
+        #self.enable_categorical = enable_categorical
         self.eval_metric = eval_metric
         self.early_stopping_rounds = early_stopping_rounds
         if kwargs:
@@ -479,7 +480,7 @@ class XgboostRegression(RegressionWorkflowBase):
                                           gpu_id=self.gpu_id,
                                           validate_parameters=self.validate_parameters,
                                           predictor=self.predictor,
-                                          enable_categorical=self.enable_categorical,
+                                          # enable_categorical=self.enable_categorical,
                                           eval_metric=self.eval_metric,
                                           early_stopping_rounds=self.early_stopping_rounds)
         self.naming = XgboostRegression.name
@@ -498,37 +499,37 @@ class XgboostRegression(RegressionWorkflowBase):
         return configuration
 
     @staticmethod
-    def _feature_importance(X: pd.DataFrame, trained_model: object, algorithm_name: str, store_path: str) -> None:
+    def _feature_importance(X: pd.DataFrame, trained_model: object, image_config: dict, algorithm_name: str, store_path: str) -> None:
         print("-----* Feature Importance *-----")
-        feature_importance(X, trained_model)
+        feature_importance(X, trained_model, image_config)
         save_fig(f"Feature Importance - {algorithm_name}", store_path)
 
     @staticmethod
-    def _histograms_feature_weights(X: pd.DataFrame, trained_model: object, algorithm_name: str, store_path: str) -> None:
-        histograms_feature_weights(X, trained_model)
-        save_fig(f"Feature Importance Score - {algorithm_name}", store_path)
+    def _histograms_feature_weights(X: pd.DataFrame, trained_model: object, image_config: dict, algorithm_name: str, store_path: str) -> None:
+        histograms_feature_weights(X, trained_model,image_config)
+        save_fig(f"Regression - {algorithm_name} - Feature Importance Score", store_path)
 
     @staticmethod
     def _permutation_importance(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame, trained_model: object,
-                                algorithm_name: str, store_path: str) -> None:
-        permutation_importance_(X, X_test, y_test, trained_model)
-        save_fig(f"Xgboost Feature Importance - {algorithm_name}", store_path)
+                                image_config: dict, algorithm_name: str, store_path: str) -> None:
+        permutation_importance_(X, X_test, y_test, trained_model, image_config)
+        save_fig(f"Regression - {algorithm_name} - Xgboost Feature Importance", store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
-        self._feature_importance(XgboostRegression.X, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
-        self._histograms_feature_weights(XgboostRegression.X, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance(XgboostRegression.X, self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._histograms_feature_weights(XgboostRegression.X, self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         self._permutation_importance(XgboostRegression.X, XgboostRegression.X_test, XgboostRegression.y_test,
-                                     self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+                                     self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
-        self._feature_importance(XgboostRegression.X, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
-        self._histograms_feature_weights(XgboostRegression.X, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importance(XgboostRegression.X, self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._histograms_feature_weights(XgboostRegression.X, self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         self._permutation_importance(XgboostRegression.X, XgboostRegression.X_test, XgboostRegression.y_test,
-                                     self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+                                     self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
 
 class DecisionTreeRegression(RegressionWorkflowBase):
@@ -713,30 +714,24 @@ class DecisionTreeRegression(RegressionWorkflowBase):
         )
         self.naming = DecisionTreeRegression.name
 
-    def plot_tree_function(self):
-        ###################################################
-        # Drawing decision tree diagrams
-        ###################################################
+    def _plot_tree_function(self, trained_model: object, image_config: dict, algorithm_name: str, store_path: str) -> None:
+        """Drawing decision tree diagrams."""
         print("-----* Decision Tree Plot *-----")
-        y = RegressionWorkflowBase().y
-        X = RegressionWorkflowBase().X
-        clf = self.model.fit(X, y)
-        plt.figure()
-        plot_tree(clf, filled=True)
-        save_fig('plot_decision_tree_regression', MODEL_OUTPUT_IMAGE_PATH)
+        decision_tree_plot(trained_model, image_config)
+        save_fig(f"Regression - {algorithm_name} - Tree Graph", store_path)
 
     @dispatch()
     def special_components(self):
-        self.plot_tree_function()
+        """Invoke all special application functions for this algorithms by Scikit-learn framework."""
+        self._plot_tree_function(self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
-        self.plot_tree_function()
+        self._plot_tree_function(self.model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
         
 
-
 class ExtraTreeRegression(RegressionWorkflowBase):
     """The automation workflow of using Extra Tree algorithm to make insightful products."""
     
@@ -942,21 +937,21 @@ class ExtraTreeRegression(RegressionWorkflowBase):
         return configuration
 
     @staticmethod
-    def _feature_importances(X_train: pd.DataFrame, trained_model: object, algorithm_name: str, store_path: str) -> None:
+    def _feature_importances(X_train: pd.DataFrame, trained_model: object,  image_config: dict, algorithm_name: str, store_path: str) -> None:
         """Draw the feature importance bar diagram."""
         print("-----* Feature Importance *-----")
-        feature_importances(X_train, trained_model)
-        save_fig(f"Feature Importance - {algorithm_name}", store_path)
+        feature_importances(X_train, trained_model, image_config)
+        save_fig(f"Regression - {algorithm_name} - Feature Importance Plot", store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
-        self._feature_importances(ExtraTreeRegression.X_train, self.model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importances(ExtraTreeRegression.X_train, self.model,  self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
-        self._feature_importances(ExtraTreeRegression.X_train, self.auto_model, self.naming, MODEL_OUTPUT_IMAGE_PATH)
+        self._feature_importances(ExtraTreeRegression.X_train, self.auto_model, self.image_config, self.naming, MODEL_OUTPUT_IMAGE_PATH)
 
 
 class RandomForestRegression(RegressionWorkflowBase):
@@ -1165,23 +1160,36 @@ class RandomForestRegression(RegressionWorkflowBase):
 
     @staticmethod
     def _feature_importances(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame,
-                             trained_model: object, algorithm_name: str, store_path: str) -> None:
+                             trained_model: object,  image_config: dict, algorithm_name: str, store_path: str) -> None:
         print("-----* Feature Importance *-----")
-        feature_importance__(X, X_test, y_test, trained_model)
-        save_fig(f"Feature Importance - {algorithm_name}", store_path)
+        feature_importance__(X, X_test, y_test, trained_model, image_config)
+        save_fig(f"Regression - {algorithm_name} - Feature Importance", store_path)
+
+    @staticmethod
+    def _box_plot(X: pd.DataFrame, X_test: pd.DataFrame, y_test: pd.DataFrame,
+                 trained_model: object,  image_config: dict, algorithm_name: str, store_path: str) -> None:
+        print("-----* Box Plot *-----")
+        box_plot(X, X_test, y_test, trained_model, image_config)
+        save_fig(f"Regression - {algorithm_name} - Box Plot", store_path)
 
     @dispatch()
     def special_components(self, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by Scikit-learn framework."""
         self._feature_importances(RandomForestRegression.X, RandomForestRegression.X_test,
-                                  RandomForestRegression.y_test, self.model, self.naming,
+                                  RandomForestRegression.y_test, self.model, self.image_config, self.naming,
+                                  MODEL_OUTPUT_IMAGE_PATH)
+        self._box_plot(RandomForestRegression.X, RandomForestRegression.X_test,
+                                  RandomForestRegression.y_test, self.model, self.image_config, self.naming,
                                   MODEL_OUTPUT_IMAGE_PATH)
 
     @dispatch(bool)
     def special_components(self, is_automl: bool = False, **kwargs) -> None:
         """Invoke all special application functions for this algorithms by FLAML framework."""
         self._feature_importances(RandomForestRegression.X, RandomForestRegression.X_test,
-                                  RandomForestRegression.y_test, self.auto_model, self.naming,
+                                  RandomForestRegression.y_test, self.auto_model, self.image_config, self.naming,
+                                  MODEL_OUTPUT_IMAGE_PATH)
+        self._box_plot(RandomForestRegression.X, RandomForestRegression.X_test,
+                                  RandomForestRegression.y_test, self.auto_model, self.image_config, self.naming,
                                   MODEL_OUTPUT_IMAGE_PATH)
 
 
