@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import matplotlib.pyplot as plt
+import pandas as pd
 from ..utils.base import save_fig
 from ..global_variable import MAP_IMAGE_PATH
 import logging
@@ -9,11 +10,16 @@ from mpl_toolkits.axes_grid1 import make_axes_locatable
 logging.captureWarnings(True)
 
 
-def map_projected(col, df):
-    """
+def map_projected(col: pd.Series, df: pd.DataFrame) -> None:
+    """Project an element data into world map.
 
-    :param col: pd.Series, one selected column from the data sheet
-    :param df: pd.DataFrame, the data sheet
+    Parameters
+    ----------
+    col : pd.Series
+        One selected column from the data sheet.
+
+    df : pd.DataFrame
+        The data sheet.
     """
     # Create point geometries
     geometry = geopandas.points_from_xy(df['LONGITUDE'], df['LATITUDE'])
@@ -27,4 +33,4 @@ def map_projected(col, df):
     world.plot(ax=ax, alpha=0.4, color='grey', edgecolor='black')
     geo_df.plot(col.name, ax=ax, s=20, cmap='gist_heat_r', cax=cax, legend=True)
     plt.title('colorbar')
-    save_fig(f"map projection - {col.name}", MAP_IMAGE_PATH)
+    save_fig(f"Map Projection - {col.name}", MAP_IMAGE_PATH)
